package com.example;

import java.util.function.UnaryOperator;

import javafx.scene.control.TextFormatter.Change;

public class NonNegativeIntegerFilter implements UnaryOperator<Change> {

	private int numberOfDigits;
	
	public NonNegativeIntegerFilter() {
		this(9);
	}
	
	public NonNegativeIntegerFilter(int numberOfDigits) {
		this.numberOfDigits = numberOfDigits;
	}
	
	@Override
	public Change apply(Change t) {
		if(t.isAdded()) {
			StringBuilder added = new StringBuilder(t.getText());
			for(int i = added.length() - 1; i >= 0; i--) {
				if(added.charAt(i) == '.') {
					added.delete(i, added.length());
				} else if(added.charAt(i) < '0' || added.charAt(i) > '9') {
					added.deleteCharAt(i);
				}
			}
			if(added.length() == 0) {
				return null;
			}
			t.setText(added.toString());

			if(getNumberOfDigits(t.getControlNewText()) > numberOfDigits) {
				return null;
			}
		} else if(t.isDeleted() && (t.getRangeEnd() - t.getRangeStart() == 1)) {
			if(t.getControlText().charAt(t.getRangeStart()) == ',') {
				t.setRange(t.getRangeStart(), t.getRangeEnd() + 1);
			}
		}
		if(t.isContentChange()) {
			int caretPosition = t.getCaretPosition();
			StringBuilder text = new StringBuilder(t.getControlNewText());
			int digitCount = 0;
			int digitStartIndex = 0;
			if(text.length() > 0 && text.charAt(0) == ',') {
				digitStartIndex = 1;
			}
			for(int i = text.length() - 1; i >= 0; i--) {
				char ch = text.charAt(i);
				if('0' <= ch && ch <= '9') {
					if(++digitCount % 3 == 0 && i > digitStartIndex) {
						text.insert(i, ',');
						if(i < caretPosition) {
							caretPosition++;
						}
					}
				} else if(ch == ',') {
					text.deleteCharAt(i);
					if(i < caretPosition) {
						caretPosition--;
					}
				}
			}
			t.setText(text.toString());
			t.setRange(0, t.getControlText().length());
			t.setAnchor(caretPosition);
			t.setCaretPosition(caretPosition);
		} else {
			if(t.getSelection().getLength() == 0) {
				if(t.getCaretPosition() == t.getControlCaretPosition()) {
					if(t.getCaretPosition() > 0
							&& t.getControlNewText().charAt(t.getCaretPosition() - 1) == ',') {
						t.setCaretPosition(t.getCaretPosition() - 2);
					}
				} else if(t.getCaretPosition() < t.getControlCaretPosition()) {
					if(t.getCaretPosition() > 0
							&& t.getControlNewText().charAt(t.getCaretPosition() - 1) == ',') {
						t.setCaretPosition(t.getCaretPosition() - 1);
					}
				} else if(t.getCaretPosition() > t.getControlCaretPosition()) {
					if(t.getCaretPosition() < t.getControlNewText().length()
							&& t.getControlNewText().charAt(t.getControlCaretPosition()) == ',') {
						t.setCaretPosition(t.getCaretPosition() + 1);
					}
				}
				if(t.getAnchor() != t.getCaretPosition()) {
					t.selectRange(t.getCaretPosition(), t.getCaretPosition());
				}
			} else {
				int offset = t.getCaretPosition() > t.getAnchor() ? 1 : 0;
				if(t.getControlNewText().charAt(t.getCaretPosition() - offset) == ',') {
					int move = t.getCaretPosition() > t.getControlCaretPosition() ? +1 : -1;
					t.selectRange(t.getAnchor(), t.getCaretPosition() + move);
				}
				if(t.getAnchor() < t.getCaretPosition()) {
					if(t.getControlNewText().charAt(t.getAnchor()) == ',') {
						t.selectRange(t.getAnchor() + 1, t.getCaretPosition());
					}
				} else {
					if(t.getAnchor() > 0 && t.getControlNewText().charAt(t.getAnchor() - 1) == ',') {
						t.selectRange(t.getAnchor() - 1, t.getCaretPosition());
					}
				}
			}
		}
		return t;
	}
	
	private static int getNumberOfDigits(CharSequence s) {
		int num = 0;
		int length = s.length();
		for(int i = 0; i < length; i++) {
			char ch = s.charAt(i);
			if('0' <= ch && ch <= '9') {
				num++;
			}
		}
		return num;
	}
}
